/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2004  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

#ifndef GobWarpOutTrail_Header
#define GobWarpOutTrail_Header

#include "melee/mship.h" //for Ship class
#include "melee/WarpOutTrail.h"
#include "games/ggob.h"

#include <string>

/**
 * 
 */
class GobWarpEffect : public PhaserEffect {
protected:
  std::string targetSystem;
  GobGame * gobGame;
  
public:
  GobWarpEffect(const std::string & targetSystem, GobGame * gobGame);
  virtual ~GobWarpEffect(){}
  /**
   * Removes the ship from play.
   */
  virtual void startEffect();
  
  /**
   * Moves the ship to the star system specified in the constructor.
   */ 
  virtual void endEffect();
};

/**
 * Used when the player warps out of one system and into another.
 */  
class GobWarpOutTrail : public Phaser {

protected:

  /**
   * Creates a warp out trail.  At the begining of this effect, the ship is
   * removed from its current star system.  At the end of this effect, the ship
   * is placed in the target star system.
   * 
   * @param targetSystem the name of the star system to warp to
   * @param the instance of GobGame
   * @param creator
   * @param pos
   * @param rel_pos
   * @param ship
   * @param sprite
   * @param osprite_index
   * @param ocolors
   * @param onum_colors
   * @param ofsize
   * @param steps
   * @param step_time
   * 
   * @see WarpOutTrail::WarpOutTrail()
   */ 
  GobWarpOutTrail(const std::string & targetSystem,
		  GobGame * gobGame,
		  SpaceLocation *creator, Vector2 pos, Vector2 rel_pos, 
		  SpaceSprite *sprite, int osprite_index, int *ocolors, 
		  int onum_colors, int ofsize, int steps, int step_time);

public:

  /**
   * Constructs a trail such that the player will end up at the specified 
   * system at the end of the effect.
   * 
   * @param the ship to move to the star system
   * @param system the name of the star system the player wishes to travel to.
   * @param the instance of GobGame
   */ 
  static GobWarpOutTrail * getGobWarpOutTrail(Ship * ship, 
	const std::string & targetSystem,
	GobGame * gobGame);

};


#endif
