/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2005  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/


#include "eventmanager.h"
#include "gameevent.h"
#include "gameaction.h"
#include "melee/mframe.h"
#include "games/ggob.h"
#include "melee/mship.h"
#include "melee/mview.h"
#include "melee/mcontrol.h"

#include <map>
#include <iostream>
#include "scp.h"
#include "melee/mhelpers.h"


static std::map<std::string,std::string> g_string_flag_map;
static std::map<std::string,double> g_double_flag_map;

void GameAction::clean()
{
  STACKTRACE;
  g_string_flag_map.clear();
  g_double_flag_map.clear();
}
void GameAction::add_system(const std::string& name, double x, double y)
{
  STACKTRACE;
  gobgame->add_system(name, (int)x, (int)y);
  gobgame->prepare();
}

int GameAction::add_planet(const std::string& system, const std::string& name, double x, double y)
{
  STACKTRACE;
  return gobgame->add_planet(system, name, (int)x, (int)y);
}

int GameAction::add_orbiter_station(const std::string& system, 
			     const std::string& planet, 
			     const std::string& station_sprite,
			     const std::string& background_picture,
			     const std::string& buildtype,
			     const std::string& commander_dialog)
{
  STACKTRACE;
  return gobgame->add_orbiter_station(system, planet, station_sprite, background_picture, buildtype,commander_dialog);
}

int GameAction::add_asteroid(const std::string& system, double x, double y, double oangle, double speed)
{
  STACKTRACE;
  return gobgame->add_asteroid(system, Vector2(x,y), (int)oangle, (int)speed);
}

int GameAction::add_player(const std::string& system, const std::string& shptype, double x, double y)
{
  STACKTRACE;
  return gobgame->add_player(system, shptype, (int)x, (int)y);
}

int GameAction::add_ship(const std::string& system, const std::string& shptype, double x, double y, double team)
{
  STACKTRACE;
  return gobgame->add_ship(system, shptype, Vector2(x,y), team);
}

double GameAction::get_player_ship_id()
{
  STACKTRACE;
  if(gobgame->gobplayer.ship)
    {
      return gobgame->gobplayer.ship->get_id();
    }
  else
    {
      tw_error("Player doesn't have ship!!!");
      return -1;
    }
}

std::string GameAction::get_player_star_system()
{
  STACKTRACE;
  return gobgame->_name;
}

void GameAction::save_flag(const std::string& name, const std::string& value)
{
  STACKTRACE;
  g_string_flag_map[name] = value;
  if(g_string_flag_map.find(name)==g_string_flag_map.end())
  {
    tw_error( (std::string("Unable to find global string flag!!: ") + name).c_str());
  }
}

std::string GameAction::get_string_flag(const std::string& name)
{
  STACKTRACE;
  if(g_string_flag_map.find(name)==g_string_flag_map.end())
  {
    tw_error( (std::string("Unable to find global string flag: ") + name).c_str());
  }
  return g_string_flag_map[name];
}

void GameAction::save_flag(const std::string& name, double value)
{
  STACKTRACE;
  g_double_flag_map[name] = value;
}

double GameAction::get_double_flag(const std::string& name)
{
  STACKTRACE;
  if(g_double_flag_map.find(name)==g_double_flag_map.end())
  {
    tw_error( (std::string("Unable to find global double flag: ") + name).c_str());
  }
  return g_double_flag_map[name];
}


void GameAction::set_target(double who, double to)
{
  STACKTRACE;
  SpaceLocation* l  =  twl::get_location((int)who);
  SpaceLocation* l2  =  twl::get_location((int)to);
  
  if(l&&l->isShip()&&l2)
    {
      ((Ship*)l)->control->target = (SpaceObject*)l2;
    }
}

void GameAction::disable_weapon(double who)
{
  STACKTRACE;
  SpaceLocation* l  =  twl::get_location((int)who);
  if(l->isShip())
    {
      ((Ship*)l)->disableWeapon();
    }
}

void GameAction::enable_weapon(double who)
{
  STACKTRACE;
  SpaceLocation* l  =  twl::get_location((int)who);
  if(l->isShip())
    {
      ((Ship*)l)->enableWeapon();
    }
}

template <class T>
int add_point_based_event(double who, double x, double y, double dist)
{
  SpaceLocation* l   =  twl::get_location((int)who);
  T* e = new T();
  boost::shared_ptr<TGameEvent> ev( e );
  int ret = e->setUp(l, Vector2(x,y), dist);
  EventManager::g_events.push_back(ev);
  return ret;
}

int GameAction::go_to_point(double who, double x, double y, double dist)
{
  STACKTRACE;
  return add_point_based_event<TEventOrderMoveToPoint>(who, x, y, dist);
}

template <class T>
int add_location_based_event(double who, double to, double dist)
{
  SpaceLocation* l1  =  twl::get_location((int)who);
  SpaceLocation* l2  =  twl::get_location((int)to);
  
  T* e = new T();
  boost::shared_ptr<TGameEvent> ev( e );
  int ret = e->setUp(l1, l2, dist);
  EventManager::g_events.push_back(ev);

  return ret;
}

int GameAction::go_to_location(double who, double to, double dist)
{
  STACKTRACE;
  return add_location_based_event<TEventOrderMoveToLocation>(who, to, dist);
}

int GameAction::guard_location(double who, double to, double dist)
{
  STACKTRACE;
  return add_location_based_event<TEventGuardLocation>(who, to, dist);
}

int GameAction::guard_point(double who, double x, double y, double dist)
{
  STACKTRACE;
  return add_point_based_event<TEventGuardPoint>(who, x, y, dist);
}

void GameAction::set_ship_team(double who, double team)
{
  STACKTRACE;
}

int GameAction::get_enemy_team()
{
  STACKTRACE;
  return gobgame->enemy_team;
}
int GameAction::get_player_team()
{
  STACKTRACE;
  return gobgame->player_team;
}

void GameAction::remove_location(double id)
{
  STACKTRACE;
  SpaceLocation* l  =  twl::get_location((int)id);
  if(l && l->exists())
    {
      l->die();
    }
}

void GameAction::set_crew(double id, double crew)
{
  STACKTRACE;
  SpaceLocation* l  =  twl::get_location((int)id);
  if(l && l->exists() && l->isShip())
    {
      Ship* s = (Ship*)l;
      s->crew = crew;
    }
}

void GameAction::end_game()
{
  STACKTRACE;
  gobgame->setGameDone(GAME_STATE_FINISHED);
}

void GameAction::scroll_text(const std::vector<std::string>& text, const std::string& background)
{
  STACKTRACE;
  RGB tmpPal[256];
  memcpy(tmpPal, palette_color, sizeof(RGB) * 256);
  set_palette((const RGB*)g_game_data[TW_FONT_PALETTE].dat);
  

  ::ScrollText(text,  D_FONT_ENGLISH, -1, background, "l", 2);

  videosystem->set_palette(tmpPal);
  videosystem->redraw();
}


void GameAction::print_message(const std::string& text, int time)
{
  message.print(time*1000, 2, "%s", text.c_str());
}

void GameAction::set_camera_mode(const std::string& mode)
{
  if(mode == "Hero" ||
     mode == "Enemy")
    {
      game->change_view(mode.c_str());
    }
  else
    {
      tw_error( (std::string("Requesting unknown camera mode : ") + mode).c_str());
    }
}

int GameAction::get_location_pos_x(double id)
{
	SpaceLocation* l  =  twl::get_location((int)id);
	if(l && l->exists())
	{
		return (int)l->normal_pos().x;
	}
	return -1;
}

 
int GameAction::get_location_pos_y(double id)
{
	SpaceLocation* l  =  twl::get_location((int)id);
	if(l && l->exists())
	{
		return (int)l->normal_pos().y;
	}
	return -1;
}

bool GameAction::exist(double id)
{
	SpaceLocation* l  =  twl::get_location((int)id);
	if(l && l->exists())
	{
		return true;
	}
	return false;
}
